package com.boman.web.core.utils;

import com.boman.common.core.exception.BaseException;
import com.boman.common.core.utils.StringUtils;
import com.boman.domain.Czrk;
import com.boman.domain.SysDept;
import com.boman.domain.SysRole;
import com.boman.domain.SysUser;
import java.util.List;
import java.util.Map;
import static com.boman.common.core.utils.StringUtils.isNotEmpty;
import static com.boman.common.core.utils.obj.ObjectUtils.isEmpty;
import static com.google.common.base.Strings.nullToEmpty;

/**
 * @author tjf
 * @Date: 2022/03/23/17:23
 */
public class CzrkUtils {
    /**
     * 拼接出前端需要的户籍地或者现居住地
     */
    public static void packAddr(List<Czrk> czrks) {
        if (isEmpty(czrks)) return;
        for (Czrk crk : czrks) {
            // 户籍地
            String province = nullToEmpty(crk.getProvince()), city = nullToEmpty(crk.getCity()), region = nullToEmpty(crk.getRegion()), villageTowns = nullToEmpty(crk.getVillageTowns()), village = nullToEmpty(crk.getVillage()), villageGroup = nullToEmpty(crk.getVillagerGroup()), nowIn = nullToEmpty(crk.getNowIn());
            crk.setDomicileSelect(province + city + region + villageTowns + village + villageGroup);

            // 现居住地
            String provinceCz = nullToEmpty(crk.getProvinceXjd()), cityCz = nullToEmpty(crk.getCityXjd()), regionCz = nullToEmpty(crk.getRegionXjd()), villageTownsCz = nullToEmpty(crk.getVillageTownsXjd()), villageCz = nullToEmpty(crk.getVillageXjd()), nowInCz = nullToEmpty(crk.getDomicile());
            crk.setNowInSelect(provinceCz + cityCz + regionCz + villageTownsCz + villageCz);
        }
    }


    /**
     * 人员信息拆分查询参数年龄
     *
     * @param czrk
     */
    public static void setAgeScope(Czrk czrk) {
        Map<String, Object> params = czrk.getParams();
        String ageScope = (String) params.get("age");
        if (isNotEmpty(ageScope)) {
            ageScope = ageScope.trim();
            String[] split = ageScope.split("-");
            params.put("startAge", Integer.parseInt(split[0]));
            if (split.length == 2) {
                params.put("endAge", Integer.parseInt(split[1]));
            }
        }
    }


    /**
     * 查询使用
     * 根据当前登录人判断该用户是什么角色权限字符,来进行判断该用户能查询到什么层级的数据
     * @param czrk
     * @param sysUser
     * @param type 查询常住还是户籍
     */
    public static void setQueryRole(Czrk czrk, SysUser sysUser,String type) {
        //todo 从部门中获取行政区域规划id
        SysDept dept = sysUser.getDept();
        if (dept == null){
            throw new BaseException("对不起，未获相关部门信息");
        }
        //判断该用户是什么角色权限字符
        List<SysRole> roles = sysUser.getRoles();
        if (roles != null && roles.size() > 0){
            for (SysRole role : roles) {
                if (role != null){
                     String roleKey = role.getRoleKey();
                     if (StringUtils.isNotBlank(roleKey)){
                         switch (roleKey){
                             case "province":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setProvinceId();
                                 }else {
                                     //czrk.setProvinceIdXjd();
                                 }
                             case "city":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setCityId();
                                 }else {
                                     //czrk.setCityIdXjd();
                                 }
                             case "region":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setRegionId();
                                 }else {
                                     //czrk.setRegionIdXjd();
                                 }
                             case "villageTowns":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setVillageTownsId();
                                 }else {
                                     //czrk.setVillageTownsXjd();
                                 }
                             case "village":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setVillageId();
                                 }else {
                                     //czrk.setVillageIdXjd();
                                 }
                             case "villagerGroup":
                                 if (type.equals(Czrk.HJ)){
                                     //czrk.setVillagerGroupId();
                                 }else {
                                     //czrk.setVillagerGroupIdXjd();
                                 }
                             default:
                         }
                     }
                }
            }
        }
    }


    /**
     * 新增用户的时候,判断登录用户是否能新增
     * @param czrk
     * @param sysUser
     */
    public static Boolean setInsertRole(Czrk czrk, SysUser sysUser) {
        Boolean flag = false;
        //todo 从部门中获取行政区域规划id
        SysDept dept = sysUser.getDept();
        if (dept == null){
            throw new BaseException("对不起，未获相关部门信息");
        }
        //判断该用户是什么角色权限字符
        List<SysRole> roles = sysUser.getRoles();
        if (roles != null && roles.size() > 0){
            for (SysRole role : roles) {
                if (role != null){
                    String roleKey = role.getRoleKey();
                    if (StringUtils.isNotBlank(roleKey)){
                        switch (roleKey){
                            case "province":
                                String provinceId = czrk.getProvinceId();
/*                                if (StringUtils.isNotBlank(provinceId) && provinceId.equals()){
                                     flag = true;
                                }*/
                            case "city":
                                String cityId = czrk.getCityId();
/*                                if (StringUtils.isNotBlank(cityId) && cityId.equals()){
                                    flag = true;
                                }*/
                            case "region":
                                String regionId = czrk.getRegionId();
/*                                if (StringUtils.isNotBlank(regionId) && regionId.equals()){
                                    flag = true;
                                }*/
                            case "villageTowns":
                                String villageTownsId = czrk.getVillageTownsId();
/*                                if (StringUtils.isNotBlank(villageTownsId) && villageTownsId.equals()){
                                    flag = true;
                                }*/
                            case "village":
                                String village = czrk.getVillage();
/*                                if (StringUtils.isNotBlank(village) && village.equals()){
                                    flag = true;
                                }*/
                            case "villagerGroup":
                                String villagerGroup = czrk.getVillagerGroup();
/*                                if (StringUtils.isNotBlank(villagerGroup) && villagerGroup.equals()){
                                    flag = true;
                                }*/
                            default:
                        }
                    }
                }
            }
        }
        return flag;
    }
}
